@@ -26,7 +26,7 @@ typedef struct SilcServerBackupStruct *SilcServerBackup;
 
 /* Callback function that is called after the key exchange and connection
    authentication protocols has been completed with a remote router. The
-   `server_entry' is the remote router entry. */
+   `server_entry' is the remote router entry or NULL on error. */
 typedef void (*SilcServerConnectRouterCallback)(SilcServer server,
 						SilcServerEntry server_entry,
 						void *context);
@@ -98,7 +98,16 @@ do {								\
   silc_server_send_notify(server, sock, FALSE, 			\
 			  type, 1, __fmt__, strlen(__fmt__));	\
   silc_free(__fmt__);						\
-} while(0);
+} while(0)
+
+/* Send notify to operators */
+#define SILC_SERVER_SEND_OPERS(server, route, local, type, fmt)		\
+do {									\
+  char *__fmt__ = silc_format fmt;					\
+  silc_server_send_opers_notify(server, route, local,			\
+			        type, 1, __fmt__, strlen(__fmt__));	\
+  silc_free(__fmt__);							\
+} while(0)
 
 /* Check whether rekey protocol is active */
 #define SILC_SERVER_IS_REKEY(sock)					\
@@ -176,11 +185,13 @@ void silc_server_announce_get_channel_topic(SilcServer server,
 					    SilcBuffer *topic);
 void silc_server_announce_get_channel_users(SilcServer server,
 					    SilcChannelEntry channel,
+					    SilcBuffer *channel_modes,
 					    SilcBuffer *channel_users,
 					    SilcBuffer *channel_users_modes);
 void silc_server_announce_get_channels(SilcServer server,
 				       SilcIDList id_list,
 				       SilcBuffer *channels,
+				       SilcBuffer **channel_modes,
 				       SilcBuffer *channel_users,
 				       SilcBuffer **channel_users_modes,
 				       SilcUInt32 *channel_users_modes_c,
@@ -196,7 +207,7 @@ void silc_server_announce_clients(SilcServer server,
 void silc_server_announce_channels(SilcServer server,
 				   unsigned long creation_time,
 				   SilcSocketConnection remote);
-void silc_server_get_users_on_channel(SilcServer server,
+bool silc_server_get_users_on_channel(SilcServer server,
 				      SilcChannelEntry channel,
 				      SilcBuffer *user_list,
 				      SilcBuffer *mode_list,
