@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: serverconfig.c,v 1.55 2002/06/15 16:06:55 priikone Exp $ */
+/* $Id: serverconfig.c,v 1.59 2002/06/25 21:43:19 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -122,9 +122,10 @@ static bool my_parse_authdata(SilcAuthMethod auth_meth, char *p,
     /* p is a plain text password */
     if (auth_data && auth_data_len) {
       if (!silc_utf8_valid(p, strlen(p))) {
-	*auth_data_len = silc_utf8_encoded_len(p, strlen(p), 0);
+	*auth_data_len = silc_utf8_encoded_len(p, strlen(p),
+					       SILC_STRING_LANGUAGE);
 	*auth_data = silc_calloc(*auth_data_len, sizeof(unsigned char));
-	silc_utf8_encode(p, strlen(p), SILC_STRING_ASCII, *auth_data,
+	silc_utf8_encode(p, strlen(p), SILC_STRING_LANGUAGE, *auth_data,
 			 *auth_data_len);
       } else {
 	*auth_data = (void *) strdup(p);
@@ -1314,6 +1315,10 @@ void silc_server_config_destroy(SilcServerConfig config)
     silc_free(config->logging_errors->file);
   if (config->logging_fatals)
     silc_free(config->logging_fatals->file);
+  silc_free(config->logging_info);
+  silc_free(config->logging_warnings);
+  silc_free(config->logging_errors);
+  silc_free(config->logging_fatals);
 
   /* Destroy the ServerInfo struct */
   if (config->server_info) {
@@ -1338,6 +1343,7 @@ void silc_server_config_destroy(SilcServerConfig config)
     silc_free(si->pid_file);
     silc_pkcs_public_key_free(si->public_key);
     silc_pkcs_private_key_free(si->private_key);
+    silc_free(si);
   }
 
   /* Now let's destroy the lists */
@@ -1847,3 +1853,31 @@ silc_server_config_get_primary_router(SilcServer server)
 
   return NULL;
 }
+
+/* If we have backup router configured that is going to replace us this
+   function returns it. */
+
+SilcServerConfigRouter *
+silc_server_config_get_backup_router(SilcServer server)
+{
+  SilcServerConfig config = server->config;
+  SilcServerConfigRouter *serv = NULL;
+  int i;
+
+  if (server->server_type != SILC_ROUTER)
+    return NULL;
+
+  serv = config->routers;
+  for (i = 0; serv; i++) {
+    if (serv->initiator == FALSE && serv->backup_router == TRUE &&
+	serv->backup_local == TRUE &&
+	!strcmp(server->config->server_info->primary->server_ip,
+		serv->backup_replace_ip) &&
+	server->config->server_info->primary->port ==
+	serv->backup_replace_port)
+      return serv;
+    serv = serv->next;
+  }
+
+  return NULL;
+}
